function [dtw_mindistance] = dtw_min(sequenceA,sequenceB)
% Description:
%       This function returns the minimum DTW value possible, also
%       utilizing circularily shifting of the cycles.
% Input:
%       sequenceA - a gait cycle vector
%       sequenceB - a gait cycle vector
% Output:
%       dtw_mindistance - the minimal DTW distance after comparing shifts.
%
% Author:
%       Bendik B. Mjaaland, 2009
%       Norwegian University of Science and Technology (NTNU)
%       
% Disclaimer:
%       This piece of software is not copyrighted, and anyone are free to
%       use it in any context as long as the author information and
%       disclaimer is preserved.


%Determine peaks
maxtabA     = peakdet(sequenceA,0.1);
maxtabB     = peakdet(sequenceB,0.1);

%Remove peaks that are below threshold
thresholdA  = 0.9*(max(sequenceA) + mean(sequenceA) ) / 2;
thresholdB  = 0.9*(max(sequenceB) + mean(sequenceB) ) / 2;

removed = 0;
for i=1:length(maxtabA(:,1))
    if maxtabA(i-removed,2) < thresholdA
        maxtabA(i-removed,:) = [];
        removed = removed + 1;
    end
end
removed = 0;
for i=1:length(maxtabB(:,1))
    if maxtabB(i-removed,2) < thresholdB
        maxtabB(i-removed,:) = [];
        removed = removed + 1;
    end
end

%Reduce to max four peaks
while(length(maxtabA(:,1)) > 4)
    index = find(maxtabA(:,2) == min(maxtabA(:,2)));
    maxtabA(index,:) = [];
end
while(length(maxtabB(:,1)) > 4)
    index = find(maxtabB(:,2) == min(maxtabB(:,2)));
    maxtabB(index,:) = [];
end

%The sequence with the smallest number of peaks should be shifted, and will
%be referred to as cosequence, while the "still" one is the basesequence.
if(length(maxtabA) > length(maxtabB))
    basesequence    = sequenceA;
    cosequence      = sequenceB;
    rotationindex   = find(maxtabA(:,2) == max(maxtabA(:,2)));
    rotationpoint   = maxtabA(rotationindex,1);
    cotab           = maxtabB;
else
    basesequence    = sequenceB;
    cosequence      = sequenceA;
    rotationindex   = find(maxtabB(:,2) == max(maxtabB(:,2)));
    rotationpoint   = maxtabB(rotationindex,1);
    cotab           = maxtabA;
end

%Determine rotation point, and make sure it is reachable
if(rotationpoint > length(cosequence))
    %Shift base sequence and rotation point backwards
    basesequence = circshift(basesequence,[1, -50]);
    rotationpoint = rotationpoint -50;
end

%Begin with the original DTW, and compare to shifting versions
dtw_mindistance = dtw(sequenceA,sequenceB); %Original DTW

for i = 1:length(cotab(:,1))
    %How many sample indexes to shift
    shift              = rotationpoint - cotab(i,1);
    new_cosequence     = circshift(cosequence,[1,shift]);
    %Calculate DTW between the basesequence, and the shifted cosequence.
    currentDTW = dtw(basesequence,new_cosequence);
    %Update minimum
    if currentDTW < dtw_mindistance
        dtw_mindistance = currentDTW;
    end
end



























