function [dtw_pairavg_matrix,dtw_std_matrix,genuinevector,impostorvector] = dtw_pairavg(templatefolder_filename,n_persons,skip)
% Description:
%       This function computes intra-person DTW matrix for every person
%       against eachother in terms of average DTW distance.
%       Hence, d(i,j) in this matrix will be the average distance between
%       person i and j.
%       Also, the method returns the DTW comparisons in terms of a genuine
%       and impostor vector, which can be used to calculate a DET curve
%       later.
% Input:
%       templatefolder_filename - root folder as input, including the
%       beginning of the subfolders for each user, such as 'TS/TS' where
%       TSxx are the subfolders, so person 6 is 'TS/TS06'.
%       n_persons - how many persons to process
%       skip - how many persons to skip (e.g. skip = 10 will result in
%       all persons in the interval <skip,n_persons] to be processed.
% Output:
%       dtw_pairavg_matrix - a matrix where cell (i,j) represents the DTW
%       distance between person i and person j.
%       dtw_std_matrix - a matrix where cell (i,j) represents the standard
%       deviation in the DTW distance between person i and person j.
%       genuinevector - all genuine comparisons
%       impostorvector - all fraudulent comparisons
%
% Author:
%       Bendik B. Mjaaland, 2009
%       Norwegian University of Science and Technology (NTNU)
%       
% Disclaimer:
%       This piece of software is not copyrighted, and anyone are free to
%       use it in any context as long as the author information and
%       disclaimer is preserved.

strcat('Starting: ',datestr(now))

%Holders and settings for DTW
dtw_pairavg_matrix = zeros(n_persons,n_persons);
dtw_std_matrix = zeros(n_persons,n_persons);
%Vectors to be saved for later: genuine and impostor comparisons. Can be
%loaded later to make DET curve.
genuinevector = [];
impostorvector = [];

%Load all templates into matlab
for i = 1+skip:n_persons
    if i < 10
        templatefolder = strcat(templatefolder_filename,'0',int2str(i));
    else
        templatefolder = strcat(templatefolder_filename,int2str(i));
    end
    %We only want templates
    query  = strcat(templatefolder,'/t_*.dat');
    %Fetch template info
    files   = dir(query);
    namount = length(files);
    %Load
    for j=1:namount
        load(strcat(templatefolder,'/',files(j).name));
    end
    %Some optional printing
    if (mod(i,10) == 0)
        strcat('Loaded templates up to test subject #',int2str(i))
    end
end

%Construct table
for i = 1+skip:n_persons
    %Find person identifier for test subject i
    if (i < 10)
        person_i = strcat('TS0',int2str(i));
    else
        person_i = strcat('TS',int2str(i));
    end
    %Find average distance from person i to all other (j)'
    for j = i:n_persons
        %Find person identifier for test subject j
        if (j < 10)
            person_j = strcat('TS0',int2str(j));
        else
            person_j = strcat('TS',int2str(j));
        end
        
        %Reset pairdtw. Pairdtw holds comparison of two people: 10 x 10
        %templates.
        pairdtw         = zeros(10,10);
        vector          = zeros(1,100);
        %Start comparing walks between person i and j
        for walk_i = 1:10
            %Find walk identifier for person i
            if (walk_i < 10)
                walk_ix = strcat('0',int2str(walk_i));
            else
                walk_ix = int2str(walk_i);
            end
            for walk_j = 1:10
                %Find walk identifier for person j
                if (walk_j < 10)
                    walk_jx = strcat('0',int2str(walk_j));
                else
                    walk_jx = int2str(walk_j);
                end
                %Calculate DTW distances. The resulting pairdtw matrix is a
                %10x10 distance table comparing person i (rows) to person
                %j (columns).
                pairdtw(walk_i,walk_j)    = dtw_min(eval(strcat('t_walk',walk_ix,'_',person_i)),eval(strcat('t_walk',walk_jx,'_',person_j)));
            end
        end        
        %Find the grand mean and std between person i and j and store in
        %the dtw_pairavg_matrix.dtw. 
        vector(1,:)             = pairdtw(:);
        dtw_pairavg_matrix(i,j) = mean(vector);
        dtw_std_matrix(i,j)     = std(vector);
        save('dtw_pairmatrix.mat','dtw_pairavg_matrix');
        save('dtw_std_matrix.mat','dtw_std_matrix');
        
        %Also update genuine / impostorvector
        if(j==i)
            genuinevector = [genuinevector,vector];
        else
            impostorvector = [impostorvector,vector];
        end
        
    end
    strcat('Done with test subject #',int2str(i))
end

%Save impostor- and genuinevector
save('TS/genuinevector.mat','genuinevector');
save('TS/impostorvector.mat','impostorvector');

strcat('ending: ',datestr(now))