function [normalizedcycles] = normalize_cycles(cycles,resultant)
% Description:
%       This function returns a complete table of the cycles of a gait signal,
%       normalized to equal lengths, provided the cycles table from the
%       detect_cycles function, and the % gaitdata resultant one-column
%       table (array).
% Input:
%       resultant - acceleration amplitudes corresponding to time
%       cycles - starting points for cycles, index-wise. Produced by the
%       function detect_cycles.
% Output:
%       normalizedcycles - complete cycle data, normalized to equal
%       lengths. The rows in the table represent gait cycles, and the
%       columns represent resultant values. Size example: 20 x 110
%
% Author:
%       Bendik B. Mjaaland, 2009
%       Norwegian University of Science and Technology (NTNU)
%       
% Disclaimer:
%       This piece of software is not copyrighted, and anyone are free to
%       use it in any context as long as the author information and
%       disclaimer is preserved.

%Collect cycle characteristics and prepare holders
[avglength, maxlength] = pairdistance(transpose(cycles));
fullcycles             = zeros(length(cycles)-1,maxlength);
normalizedcycles       = zeros(length(cycles)-1,avglength);

%Desired timeline
steps                  = 0.01:0.01:avglength/100;

%Fill fullcycles with all discovered cycles
for i=1:length(cycles)-1
    start           = cycles(i,1);
    stop            = cycles(i+1,1);
    fullcycles(i,1:stop-start) = resultant(start:(stop-1),1);
end

%Interpolate cycles
for i = 1:length(cycles)-1
    %Get original length   
    oldlength     = cycles(i+1,1) - cycles(i,1);
    %Interpolation neccesary?
    if(oldlength == avglength)
        %Keep original
        normalizedcycles(i,:) = fullcycles(i,1:oldlength);
        continue;
    end
    
    %Skew original steps so that the number of steps remain unchanged,
    %but also such that the scale is increased / decreased towards average.
    oldsteps      = 0.01:(avglength/oldlength)/100:avglength/100;
    %Set endpoint
    oldsteps(1,oldlength) = avglength/100;
    %Interpolate based on the skewed old steps and the desired ones.
    normalizedcycles(i,:) = interp1(oldsteps,fullcycles(i,1:oldlength), steps);
end
