function [remainingcycles] = omit_cycles(normalizedcycles)
% Description:
%       This function uses DTW scores to remove outliers from a set of
%       normalized cycles.
% Input:
%       normalizedcycles - complete cycle data, normalized to equal
%       lengths. The rows in the table represent gait cycles, and the
%       columns represent resultant values. Size example: 20 x 110
% Output:
%       remaining - complete cycle data, normalized, with outliers removed.
%
% Author:
%       Bendik B. Mjaaland, 2009
%       Norwegian University of Science and Technology (NTNU)
%       
% Disclaimer:
%       This piece of software is not copyrighted, and anyone are free to
%       use it in any context as long as the author information and
%       disclaimer is preserved.

%Gait cycles within the range [avg_dtw +/- threshold percentage] will be
%kept. The threshold is sliding, decreasing at every run except the last.
thresholds        = [0.7, 0.5, 0.3, 0.2];
binarycycles      = ones(1,length(normalizedcycles(:,1)));

% % Temporary logging for determining how many cycle skips are made
% lengths = fopen('omittingreport.txt','a');
% fprintf(lengths,'------');
% fprintf(lengths,'\r\n');

%Calculate the DTW matrix for this job
dtwscores = zeros(length(normalizedcycles(:,1)),length(normalizedcycles(:,1)));
for i = 1:length(normalizedcycles(:,1))
   for j = i:length(normalizedcycles(:,1))
        dtwscores(i,j) = dtw(normalizedcycles(i,:),normalizedcycles(j,:));
   end
end
%Transpose and merge (symmetry)
dtwscores = dtwscores + transpose(dtwscores);

%Perform the reduction of cycles, once for each threshold.
for i = 1:length(thresholds(1,:))
    %Calculate mean and reduce degree of freedom by 1 because
    %the sum should be divided by n-1 instead of n
    means     = mean(dtwscores,2);
    means     = means * length(means) / (length(means) -1);
    totalmean = mean(means);
    
% %     Temporary logging
%     fprintf(lengths,'\r\n');
%     fprintf(lengths,int2str(length(normalizedcycles(:,1))));
%     fprintf(lengths,' -> ');

    %Mark cycles outside acceptable range
    for j = 1:length(means)
        current = means(j,1);
        if((current < (totalmean * (1-thresholds(i)))) || (current > (totalmean * (1+thresholds(i)))))
            binarycycles(1,j) = 0;
        end
    end
    
    %Remove irregular cycles
    removed = 0;
    for k = 1:length(binarycycles(1,:))
       if(binarycycles(1,k) == 0)
          normalizedcycles(k-removed,:) = [];
          dtwscores(k-removed,:)        = [];
          dtwscores(:,k-removed)        = [];
          removed                       = removed + 1;
       end
    end
    %Reset binarycycles vector
    binarycycles = ones(1,length(normalizedcycles(:,1)));
    
%     %Temporary log
%     fprintf(lengths,int2str(length(normalizedcycles(:,1))));
%     fprintf(lengths,'\r\n');
end

%Holder for remaining cycles (those not to be omited)
remainingcycles = normalizedcycles;

%%Temporary log
% fclose(lengths);